library GPHook;

{$R *.RES}

uses

  Windows,
  SysUtils,
  Messages;



// Memory map file stuff

{
  The CreateFileMapping function creates unnamed file-mapping object
  for the specified file.
}

function CreateMMF(Name: string; Size: Integer): THandle;
begin
  Result := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0, Size, PChar(Name));
  if Result <> 0 then
  begin
    if GetLastError = ERROR_ALREADY_EXISTS then
    begin
      CloseHandle(Result);
      Result := 0;
    end;
  end;
end;

{ The OpenFileMapping function opens a named file-mapping object. }

function OpenMMF(Name: string): THandle;
begin
  Result := OpenFileMapping(FILE_MAP_ALL_ACCESS, False, PChar(Name));
  // The return value is an open handle to the specified file-mapping object.
end;

{
 The MapViewOfFile function maps a view of a file into
 the address space of the calling process.
}

function MapMMF(MMFHandle: THandle): Pointer;
begin
  Result := MapViewOfFile(MMFHandle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
end;

{
  The UnmapViewOfFile function unmaps a mapped view of a file
  from the calling process's address space.
}

function UnMapMMF(P: Pointer): Boolean;
begin
  Result := UnmapViewOfFile(P);
end;

function CloseMMF(MMFHandle: THandle): Boolean;
begin
  Result := CloseHandle(MMFHandle);
end;

const
MMFName = 'MsgFilterHookDemo';

type
  PMMFData = ^TMMFData;
  TMMFData = record
    NextHook: HHOOK;
    WinHandle: HWND;
    Debug: boolean;
  end;

  // global variables, only valid in the process which installs the hook.
var
  MMFHandle: THandle;
  MMFData: PMMFData;

function UnMapAndCloseMMF: Boolean;
begin
  Result := False;
  if UnMapMMF(MMFData) then
  begin
    MMFData := nil;
    if CloseMMF(MMFHandle) then
    begin
      MMFHandle := 0;
      Result := True;
    end;
  end;
end;


const
  CM_POST_KEY = WM_USER + $1000;

function HookCallBack( Code    : Integer;
                        wParam  : WPARAM;
                        lParam  : LPARAM
                        )       : LRESULT; stdcall;
{This is the CallBack function called by the Hook}
var
  MMFHandle: THandle;
  MMFData: PMMFData;
  Debug: boolean;
begin
  Result := 0;
  MMFHandle := OpenMMF(MMFName);
  if MMFHandle <> 0 then
  begin
    MMFData := MapMMF(MMFHandle);
    if MMFData <> nil then
    begin
      Debug := MMFData.Debug;
      {if a key was pressed/released}
      if code=HC_ACTION then
      begin
        //if Debug then
        if Debug = true then OutputDebugString(pchar('GPHook: Key pressed, HC_ACTION. hWnd = '+inttostr(MMFData.WinHandle)));
        // Map keys to calling App
        PostMessage(MMFData.WinHandle,CM_POST_KEY,wParam,lParam);
      end else
      begin
        if Debug = true then OutputDebugString(pchar('GPHook: Key pressed, Not_HC_ACTION'));
      end;
      //call to next hook of the chain
      Result := CallNextHookEx(MMFData.NextHook, Code, wParam, lParam);
      UnMapMMF(MMFData);
    end;
    CloseMMF(MMFHandle);
  end;
end;


procedure HookOn(WinHandle: HWND; Debug: boolean); stdcall;
begin
//  Result := False;
  if Debug = true then OutputDebugString(pchar('GPHook: HookOn. Hooking Keyboard Callback procedure. hWnd = '+inttostr(WinHandle)));
  if (MMFData = nil) and (MMFHandle = 0) then
  begin
    MMFHandle := CreateMMF(MMFName, SizeOf(TMMFData));
    if MMFHandle <> 0 then
    begin
      if Debug = true then OutputDebugString(pchar('GPHook: HookOn. MMF created.'+inttostr(MMFHandle)));
      MMFData := MapMMF(MMFHandle);
      if MMFData <> nil then
      begin
        MMFData.WinHandle := WinHandle;
        MMFData.Debug := Debug;
        MMFData.NextHook := SetWindowsHookEx(WH_KEYBOARD, @HookCallBack, HInstance , 0);
        if Debug = true then OutputDebugString(pchar('GPHook: HookOn. Hook has been set'));
        if MMFData.NextHook = 0 then
          UnMapAndCloseMMF
        else
//          Result := True;
      end
      else
      begin
        CloseMMF(MMFHandle);
        MMFHandle := 0;
      end;
    end;
  end;
end;

//procedure to uninstall the hook
procedure HookOff;  stdcall;
var
  Debug: boolean;

begin
  Debug := MMFData.Debug;

  if (MMFData <> nil) and (MMFHandle <> 0) then
    if UnHookWindowsHookEx(MMFData^.NextHook) then
        UnMapAndCloseMMF;
//      Result := UnMapAndCloseMMF;
  if Debug = true then OutputDebugString(pchar('GPHook: HookOff. Keyboard Callback procedure Unhooked.'));
end;


exports
 {Export the procedures}
  HookOn,
  HookOff;
 end.




