unit GPHookMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

const
  NameDLL       = 'GPHook.dll';
  CM_COMMAND_KEY  = WM_USER + $1000;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LoadKeyBoardHook;

  private
    { Private declarations }
    HandleDLL: THandle;
    AppHandle: HWnd;

    HookOn: procedure(PassedAppHandle: HWnd; Debug:boolean); stdcall;
    HookOff: procedure; stdcall;

    procedure HookProc(var message: TMessage); message  CM_COMMAND_KEY;


  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  Debug: boolean;

implementation

{$R *.DFM}

{ Functions to obtain handle of desired window }
function GetHandleFromWindowTitle(const titletext: string): hWnd;
var
  strbuf: Array[0..255] of Char;
begin
  result := FindWindow(PChar(0), StrPCopy(strbuf,titletext));
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  {Load GPHook.dll}
  LoadKeyBoardHook;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {Uninstall the Hook}
  if Assigned(HookOff) then
    HookOff;

  {Free the DLL}
  if HandleDLL<>0 then
    FreeLibrary(HandleDLL);
end;


// Load and Set Keyboard Hook
procedure TForm1.LoadKeyBoardHook;
begin
  //load the GPHook dll and get the handle.
  HandleDLL:=LoadLibrary( PChar(ExtractFilePath(Application.Exename)+NameDLL ) );
  if HandleDLL = 0 then
  begin
    raise Exception.Create('Can not load DLL');
  end;
  //find the address of the specified exported DLL functions.
  @HookOn :=GetProcAddress(HandleDLL, 'HookOn');
  @HookOff:=GetProcAddress(HandleDLL, 'HookOff');
  //if functions not found, raise an exception.
  if not assigned(HookOn) or not assigned(HookOff) then
  begin
    raise Exception.Create('Cannot find the required DLL functions');
  end;

  //Get the applications window handle
  AppHandle := GetHandleFromWindowTitle('GPHookDemo');

  //call the hook dll function "HookOn" to enable key hooking
  //pass application window handle and debug enable (bool) params
  HookOn(AppHandle, true);
end;


//This is the procedure called by the keyboard hook when a key action occurs
procedure TForm1.HookProc(var message: TMessage);
var
  TransBuf: PChar;
  scancode: integer;
  CurrentKeyState: TkeyboardState;
  loop: integer;
  TransChar: array[0..32] of Char;

begin
  for loop := 0 to 3 do
  begin
    //clear and null-terminate the buffer.
    TransChar[loop] := char(0);
  end;

  //pointer to char buffer
  TransBuf:=TransChar;

  //The Windows API call GetKeyboardState function copies the status of
  //the 256 virtual keys to the buffer CurrentKeyState.
  GetKeyboardState(CurrentKeyState);

  //The MapVirtualKey function translates (maps) the
  //virtual-key code (sent from the GPHook dll) into a scan code.
  scancode := MapVirtualKey(Message.WParam,1);

  //The ToAscii function translates the specified virtual-key code
  //and keyboard state to the corresponding Windows character or
  //characters. The function translates the code using the input
  //language and physical keyboard layout identified by the given
  //keyboard layout handle.
  ToAscii(Message.WParam, scancode, CurrentKeyState, TransBuf,3);

  //Show the hooked char
  Label1.Caption := TransChar;

end;


end.
